"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stmt_matcher = exports.stmts_matcher = exports.deduction_entry_matcher = exports.exp_matcher = void 0;
const Exp = __importStar(require("../exp"));
const Stmt = __importStar(require("../stmt"));
const pt = __importStar(require("../../partech"));
const ut = __importStar(require("../../ut"));
function exp_matcher(tree) {
    return pt.Tree.matcher({
        "exp:var": ({ name }) => Exp.v(pt.Tree.str(name)),
        "exp:pi": ({ name, arg_t, ret_t }) => Exp.pi(pt.Tree.str(name), exp_matcher(arg_t), exp_matcher(ret_t)),
        "exp:arrow": ({ arg_t, ret_t }) => Exp.pi("_", exp_matcher(arg_t), exp_matcher(ret_t)),
        "exp:fn": ({ name, ret }) => Exp.fn(pt.Tree.str(name), exp_matcher(ret)),
        "exp:ap": ({ target, args }) => {
            let exp = Exp.v(pt.Tree.str(target));
            for (const arg of pt.matchers.one_or_more_matcher(args)) {
                exp = Exp.ap(exp, exp_matcher(arg));
            }
            return exp;
        },
        "exp:sigma": ({ name, car_t, cdr_t }) => Exp.sigma(pt.Tree.str(name), exp_matcher(car_t), exp_matcher(cdr_t)),
        "exp:pair": ({ car_t, cdr_t }) => Exp.sigma("_", exp_matcher(car_t), exp_matcher(cdr_t)),
        "exp:cons": ({ car, cdr }) => Exp.cons(exp_matcher(car), exp_matcher(cdr)),
        "exp:car": ({ target }) => Exp.car(exp_matcher(target)),
        "exp:cdr": ({ target }) => Exp.cdr(exp_matcher(target)),
        "exp:nat": () => Exp.nat,
        "exp:zero": () => Exp.zero,
        "exp:add1": ({ prev }) => Exp.add1(exp_matcher(prev)),
        "exp:number": ({ value }, { span }) => {
            const n = Number.parseInt(pt.Tree.str(value));
            if (Number.isNaN(n)) {
                throw new pt.ParsingError(`Expecting number, instead of: ${ut.inspect(n)}`, { span });
            }
            else {
                return Exp.nat_from_number(n);
            }
        },
        "exp:nat_ind": ({ target, motive, base, step }) => Exp.nat_ind(exp_matcher(target), exp_matcher(motive), exp_matcher(base), exp_matcher(step)),
        "exp:equal": ({ t, from, to }) => Exp.equal(exp_matcher(t), exp_matcher(from), exp_matcher(to)),
        "exp:same": () => Exp.same,
        "exp:replace": ({ target, motive, base }) => Exp.replace(exp_matcher(target), exp_matcher(motive), exp_matcher(base)),
        "exp:trivial": () => Exp.trivial,
        "exp:sole": () => Exp.sole,
        "exp:absurd": () => Exp.absurd,
        "exp:absurd_ind": ({ target, motive }) => Exp.absurd_ind(exp_matcher(target), exp_matcher(motive)),
        "exp:str": () => Exp.str,
        "exp:quote": ({ value }) => {
            const str = pt.Tree.str(value);
            return Exp.quote(str.slice(1, str.length - 1));
        },
        "exp:type": () => Exp.type,
        "exp:begin": ({ stmts, ret }) => Exp.begin(stmts_matcher(stmts), exp_matcher(ret)),
        "exp:deduction": ({ deduction_entries, deduction_args }) => {
            const entries = pt.matchers
                .one_or_more_matcher(deduction_entries)
                .map(deduction_entry_matcher);
            const args = pt.matchers
                .zero_or_more_matcher(deduction_args)
                .map(exp_matcher);
            return deduction_aux(entries, args);
        },
    })(tree);
}
exports.exp_matcher = exp_matcher;
function deduction_aux(entries, args) {
    if (entries.length === 0) {
        throw new Error("entries.length === 0");
    }
    else if (entries.length === 1) {
        let [{ t, exp }] = entries;
        if (args.length === 0)
            return Exp.the(t, exp);
        for (const arg of args)
            exp = Exp.ap(exp, arg);
        return Exp.the(t, exp);
    }
    else {
        const [{ t, exp }, ...rest] = entries;
        const arg = deduction_aux(entries.slice(1), args);
        return Exp.the(t, Exp.ap(exp, arg));
    }
}
function deduction_entry_matcher(tree) {
    return pt.Tree.matcher({
        "deduction_entry:deduction_entry": ({ t, exp }) => ({
            t: exp_matcher(t),
            exp: exp_matcher(exp),
        }),
    })(tree);
}
exports.deduction_entry_matcher = deduction_entry_matcher;
function stmts_matcher(tree) {
    return pt.Tree.matcher({
        "stmts:stmts": ({ stmts }) => pt.matchers.zero_or_more_matcher(stmts).map(stmt_matcher),
    })(tree);
}
exports.stmts_matcher = stmts_matcher;
function stmt_matcher(tree) {
    return pt.Tree.matcher({
        "stmt:def": ({ name, exp }) => Stmt.def(pt.Tree.str(name), exp_matcher(exp)),
        "stmt:claim": ({ claim, t, define, exp }, { span }) => {
            if (pt.Tree.str(claim) !== pt.Tree.str(define)) {
                throw new pt.ParsingError("Name mismatch.\n" +
                    `- name to claim  : ${pt.Tree.str(claim)}\n` +
                    `- name to define : ${pt.Tree.str(define)}\n`, { span });
            }
            return Stmt.def(pt.Tree.str(claim), Exp.the(exp_matcher(t), exp_matcher(exp)));
        },
        "stmt:show": ({ exp }) => Stmt.show(exp_matcher(exp)),
    })(tree);
}
exports.stmt_matcher = stmt_matcher;
