"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.check_fn = void 0;
const Exp = __importStar(require("../exp"));
const Value = __importStar(require("../value"));
const Pattern = __importStar(require("../pattern"));
const Neutral = __importStar(require("../neutral"));
const Ctx = __importStar(require("../ctx"));
const Trace = __importStar(require("../../trace"));
const ut = __importStar(require("../../ut"));
function check_fn(mod, ctx, fn, pi) {
    const result_ctx = match_pattern(mod, ctx, fn.pattern, pi.arg_t, new Map());
    if (result_ctx === undefined)
        throw new Trace.Trace(ut.aline(`
        |Exp.check_fn -- pattern mismatch.
        |- fn: ${Exp.repr(fn)}
        |`));
    Exp.check(mod, result_ctx, fn.ret, Value.Closure.apply(pi.ret_t_cl, Value.not_yet(pi.arg_t, Neutral.v(Value.pi_arg_name(pi)))));
}
exports.check_fn = check_fn;
function match_pattern(mod, ctx, pattern, t, matched) {
    if (pattern.kind === "Pattern.v")
        return match_v(mod, ctx, pattern, t, matched);
    if (pattern.kind === "Pattern.datatype" && t.kind === "Value.type")
        return match_datatype(mod, ctx, pattern, matched);
    if (pattern.kind === "Pattern.data" &&
        t.kind === "Value.type_constructor" &&
        t.name === pattern.name)
        // TODO Why we can not normalize `type_constructor` to `datatype`?
        return match_data(mod, ctx, pattern, t, t, matched);
    if (pattern.kind === "Pattern.data" &&
        t.kind === "Value.datatype" &&
        t.type_constructor.name === pattern.name)
        return match_data(mod, ctx, pattern, t.type_constructor, t, matched);
    return undefined;
}
function match_v(mod, ctx, v, t, matched) {
    const found = matched.get(v.name);
    if (found === undefined)
        return Ctx.extend(ctx, v.name, t);
    if (Value.conversion(mod, ctx, Value.type, found, t))
        return ctx;
    return undefined;
}
function match_datatype(mod, ctx, datatype, matched) {
    // NOTE
    // - Examples:
    //   - List(T): Type
    const type_constructor = Exp.evaluate(mod, Ctx.to_env(ctx), Exp.v(datatype.name));
    if (type_constructor.kind !== "Value.type_constructor")
        throw new Trace.Trace("expecting type_constructor");
    return match_patterns(mod, ctx, datatype.args, type_constructor.t, matched);
}
function match_data(mod, ctx, data, type_constructor, t, matched) {
    // NOTE
    // - Examples:
    //   - List.null(T): List(T)
    //   - List.cons(T)(head)(tail): List(T)
    //   - Vec.null(T): Vec(T)(Nat.zero)
    //   - Vec.cons(T)(prev)(head)(tail): Vec(T)(Nat.succ(prev))
    // NOTE
    // - We will infer the type of every (nested) pattern variables.
    const data_constructor = Exp.do_dot(type_constructor, data.tag);
    if (data_constructor.kind !== "Value.data_constructor")
        throw new Trace.Trace("expecting data_constructor");
    const result_ctx = match_patterns(mod, ctx, data.args, data_constructor.t, matched);
    if (result_ctx === undefined)
        return undefined;
    // NOTE
    // - We simply do a check after the `match_patterns`,
    //   the type will not be used to constrain pattern variables.
    Exp.check(mod, result_ctx, Pattern.to_exp(data), t);
    return result_ctx;
}
// NOTE side effect on `matched`
function match_patterns(mod, ctx, patterns, t, matched) {
    if (patterns.length === 0)
        return ctx;
    let result_ctx = ctx;
    for (const pattern of patterns) {
        if (t.kind !== "Value.pi")
            return undefined;
        result_ctx = match_pattern(mod, result_ctx, pattern, t.arg_t, matched);
        if (result_ctx === undefined)
            return undefined;
        t = Value.Closure.apply(t.ret_t_cl, Exp.evaluate(mod, Ctx.to_env(result_ctx), Pattern.to_exp(pattern)));
    }
    return result_ctx;
}
