"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.infer = void 0;
const Exp = __importStar(require("../exp"));
const Value = __importStar(require("../value"));
const Ctx = __importStar(require("../ctx"));
const Trace = __importStar(require("../../trace"));
const ut = __importStar(require("../../ut"));
const exp_infer_v_1 = require("./exp-infer-v");
const exp_infer_cls_1 = require("./exp-infer-cls");
const exp_infer_pi_1 = require("./exp-infer-pi");
const exp_infer_ap_1 = require("./exp-infer-ap");
const exp_infer_dot_1 = require("./exp-infer-dot");
const exp_infer_equal_1 = require("./exp-infer-equal");
const exp_infer_replace_1 = require("./exp-infer-replace");
const exp_infer_absurd_ind_1 = require("./exp-infer-absurd-ind");
const exp_infer_union_1 = require("./exp-infer-union");
const exp_infer_type_constructor_1 = require("./exp-infer-type-constructor");
const exp_infer_begin_1 = require("./exp-infer-begin");
const exp_infer_the_1 = require("./exp-infer-the");
function infer(mod, ctx, exp) {
    try {
        if (exp.kind === "Exp.v")
            return exp_infer_v_1.infer_v(mod, ctx, exp);
        if (exp.kind === "Exp.pi")
            return exp_infer_pi_1.infer_pi(mod, ctx, exp);
        if (exp.kind === "Exp.ap")
            return exp_infer_ap_1.infer_ap(mod, ctx, exp);
        if (exp.kind === "Exp.cls")
            return exp_infer_cls_1.infer_cls(mod, ctx, exp);
        if (exp.kind === "Exp.obj" && exp.properties.size === 0)
            return Value.cls([], Value.Telescope.create(mod, Ctx.to_env(ctx), undefined, []));
        if (exp.kind === "Exp.dot")
            return exp_infer_dot_1.infer_dot(mod, ctx, exp);
        if (exp.kind === "Exp.equal")
            return exp_infer_equal_1.infer_equal(mod, ctx, exp);
        if (exp.kind === "Exp.replace")
            return exp_infer_replace_1.infer_replace(mod, ctx, exp);
        if (exp.kind === "Exp.absurd")
            return Value.type;
        if (exp.kind === "Exp.absurd_ind")
            return exp_infer_absurd_ind_1.infer_absurd_ind(mod, ctx, exp);
        if (exp.kind === "Exp.str")
            return Value.type;
        if (exp.kind === "Exp.quote")
            return Value.quote(exp.str);
        if (exp.kind === "Exp.union")
            return exp_infer_union_1.infer_union(mod, ctx, exp);
        if (exp.kind === "Exp.type_constructor")
            return exp_infer_type_constructor_1.infer_type_constructor(mod, ctx, exp);
        if (exp.kind === "Exp.type")
            return Value.type;
        if (exp.kind === "Exp.begin")
            return exp_infer_begin_1.infer_begin(mod, ctx, exp);
        if (exp.kind === "Exp.the")
            return exp_infer_the_1.infer_the(mod, ctx, exp);
        throw infer_error(exp);
    }
    catch (error) {
        if (error instanceof Trace.Trace) {
            throw Trace.trail(error, exp);
        }
        else {
            throw error;
        }
    }
}
exports.infer = infer;
function infer_error(exp) {
    let exp_repr = Exp.repr(exp);
    exp_repr = exp_repr.replace(/\s+/g, " ");
    return new Trace.Trace(ut.aline(`
       |I can not infer the type of ${exp_repr}.
       |I suggest you add a type annotation to the expression.
       |`));
}
