"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.scope_entry_matcher = exports.scope_matcher = exports.sat_entry_matcher = exports.sat_matcher = exports.property_matcher = exports.properties_matcher = exports.stmt_matcher = exports.stmts_matcher = exports.sum_entry_matcher = exports.sums_matcher = exports.top_matcher = exports.tops_matcher = exports.deduction_entry_matcher = exports.head_matcher = exports.case_entry_matcher = exports.cases_matcher = exports.exp_matcher = void 0;
const Exp = __importStar(require("../exp"));
const Pattern = __importStar(require("../pattern"));
const Top = __importStar(require("../top"));
const Stmt = __importStar(require("../stmt"));
const pt = __importStar(require("../../partech"));
function exp_matcher(tree) {
    return pt.Tree.matcher({
        "exp:var": ({ name }) => Exp.v(pt.Tree.str(name)),
        "exp:pi": ({ name, arg_t, ret_t }) => Exp.pi(pt.Tree.str(name), exp_matcher(arg_t), exp_matcher(ret_t)),
        "exp:arrow": ({ arg_t, ret_t }) => Exp.pi("_", exp_matcher(arg_t), exp_matcher(ret_t)),
        "exp:fn": ({ pattern, ret }) => Exp.fn(pattern_matcher(pattern), exp_matcher(ret)),
        "exp:case_fn": ({ cases }) => Exp.case_fn(cases_matcher(cases)),
        "exp:ap": ({ target, args }) => {
            let exp = Exp.v(pt.Tree.str(target));
            for (const arg of pt.matchers.one_or_more_matcher(args)) {
                exp = Exp.ap(exp, exp_matcher(arg));
            }
            return exp;
        },
        "exp:cls": ({ sat, scope }) => Exp.cls(sat_matcher(sat), scope_matcher(scope)),
        "exp:obj": ({ properties }) => Exp.obj(properties_matcher(properties)),
        "exp:field": ({ target, name }) => Exp.dot(head_matcher(target), pt.Tree.str(name)),
        "exp:method": ({ target, name, args }) => {
            let exp = Exp.dot(head_matcher(target), pt.Tree.str(name));
            for (const arg of pt.matchers.one_or_more_matcher(args)) {
                exp = Exp.ap(exp, exp_matcher(arg));
            }
            return exp;
        },
        "exp:equal": ({ t, from, to }) => Exp.equal(exp_matcher(t), exp_matcher(from), exp_matcher(to)),
        "exp:same": () => Exp.same,
        "exp:replace": ({ target, motive, base }) => Exp.replace(exp_matcher(target), exp_matcher(motive), exp_matcher(base)),
        "exp:absurd": () => Exp.absurd,
        "exp:absurd_ind": ({ target, motive }) => Exp.absurd_ind(exp_matcher(target), exp_matcher(motive)),
        "exp:str": () => Exp.str,
        "exp:quote": ({ value }) => {
            const str = pt.Tree.str(value);
            return Exp.quote(str.slice(1, str.length - 1));
        },
        "exp:union": ({ head, tail }) => {
            let exp = exp_matcher(head);
            for (const right of pt.matchers.zero_or_more_matcher(tail)) {
                exp = Exp.union(exp, exp_matcher(right));
            }
            return exp;
        },
        "exp:type": () => Exp.type,
        "exp:object": () => Exp.cls([], []),
        "exp:begin": ({ stmts, ret }) => Exp.begin(stmts_matcher(stmts), exp_matcher(ret)),
        "exp:deduction": ({ deduction_entries, deduction_args }) => {
            const entries = pt.matchers
                .one_or_more_matcher(deduction_entries)
                .map(deduction_entry_matcher);
            const args = pt.matchers
                .zero_or_more_matcher(deduction_args)
                .map(exp_matcher);
            return deduction_aux(entries, args);
        },
    })(tree);
}
exports.exp_matcher = exp_matcher;
function cases_matcher(tree) {
    return pt.Tree.matcher({
        "cases:cases": ({ cases }) => pt.matchers.one_or_more_matcher(cases).map(case_entry_matcher),
    })(tree);
}
exports.cases_matcher = cases_matcher;
function case_entry_matcher(tree) {
    return pt.Tree.matcher({
        "case_entry:case_entry": ({ pattern, ret }) => ({
            pattern: pattern_matcher(pattern),
            ret: exp_matcher(ret),
        }),
    })(tree);
}
exports.case_entry_matcher = case_entry_matcher;
function head_matcher(tree) {
    return pt.Tree.matcher({
        "head:var": ({ name }) => Exp.v(pt.Tree.str(name)),
        "head:ap": ({ target, args }) => {
            let exp = Exp.v(pt.Tree.str(target));
            for (const arg of pt.matchers.one_or_more_matcher(args)) {
                exp = Exp.ap(exp, exp_matcher(arg));
            }
            return exp;
        },
        "head:field": ({ target, name }) => Exp.dot(head_matcher(target), pt.Tree.str(name)),
        "head:method": ({ target, name, args }) => {
            let exp = Exp.dot(head_matcher(target), pt.Tree.str(name));
            for (const arg of pt.matchers.one_or_more_matcher(args)) {
                exp = Exp.ap(exp, exp_matcher(arg));
            }
            return exp;
        },
    })(tree);
}
exports.head_matcher = head_matcher;
function pattern_variable_p(s) {
    // NOTE The rule is "if OneCap must be ALL_CAPS"
    if (one_cap_p(s)) {
        return all_caps_p(s);
    }
    else {
        return true;
    }
}
function one_cap_p(s) {
    return s !== s.toLowerCase();
}
function all_caps_p(s) {
    return s === s.toUpperCase();
}
function pattern_matcher(tree) {
    return pt.Tree.matcher({
        "pattern:v": ({ name }) => {
            const s = pt.Tree.str(name);
            return Pattern.v(s);
            return pattern_variable_p(s)
                ? Pattern.v(s)
                : Pattern.datatype(pt.Tree.str(name), []);
        },
        "pattern:datatype": ({ name, args }) => Pattern.datatype(pt.Tree.str(name), pt.matchers.one_or_more_matcher(args).map(pattern_matcher)),
        "pattern:data": ({ name, tag, args }) => Pattern.data(pt.Tree.str(name), pt.Tree.str(tag), pt.matchers.one_or_more_matcher(args).map(pattern_matcher)),
        "pattern:data_nullary": ({ name, tag }) => Pattern.data(pt.Tree.str(name), pt.Tree.str(tag), []),
    })(tree);
}
function deduction_aux(entries, args) {
    if (entries.length === 0) {
        throw new Error("entries.length === 0");
    }
    else if (entries.length === 1) {
        let [{ t, exp }] = entries;
        if (args.length === 0)
            return Exp.the(t, exp);
        for (const arg of args)
            exp = Exp.ap(exp, arg);
        return Exp.the(t, exp);
    }
    else {
        const [{ t, exp }, ...rest] = entries;
        const arg = deduction_aux(entries.slice(1), args);
        return Exp.the(t, Exp.ap(exp, arg));
    }
}
function deduction_entry_matcher(tree) {
    return pt.Tree.matcher({
        "deduction_entry:deduction_entry": ({ t, exp }) => ({
            t: exp_matcher(t),
            exp: exp_matcher(exp),
        }),
    })(tree);
}
exports.deduction_entry_matcher = deduction_entry_matcher;
function tops_matcher(tree) {
    return pt.Tree.matcher({
        "tops:tops": ({ tops }) => pt.matchers.zero_or_more_matcher(tops).map(top_matcher),
    })(tree);
}
exports.tops_matcher = tops_matcher;
function top_matcher(tree) {
    return pt.Tree.matcher({
        "top:def": ({ name, exp }) => Top.def(pt.Tree.str(name), undefined, exp_matcher(exp)),
        "top:claim": ({ claim, t, define, exp }, { span }) => {
            if (pt.Tree.str(claim) !== pt.Tree.str(define)) {
                throw new pt.ParsingError("Name mismatch.\n" +
                    `- name to claim  : ${pt.Tree.str(claim)}\n` +
                    `- name to define : ${pt.Tree.str(define)}\n`, { span });
            }
            return Top.def(pt.Tree.str(claim), exp_matcher(t), exp_matcher(exp));
        },
        "top:show": ({ exp }) => Top.show(exp_matcher(exp)),
        "top:datatype": ({ name, t, sums }) => Top.type_constructor(Exp.type_constructor(pt.Tree.str(name), exp_matcher(t), sums_matcher(sums))),
    })(tree);
}
exports.top_matcher = top_matcher;
function sums_matcher(tree) {
    return pt.Tree.matcher({
        "sums:sums": ({ sums }) => pt.matchers.zero_or_more_matcher(sums).map(sum_entry_matcher),
    })(tree);
}
exports.sums_matcher = sums_matcher;
function sum_entry_matcher(tree) {
    return pt.Tree.matcher({
        "sum_entry:sum_entry": ({ tag, t }) => ({
            tag: pt.Tree.str(tag),
            t: exp_matcher(t),
        }),
    })(tree);
}
exports.sum_entry_matcher = sum_entry_matcher;
function stmts_matcher(tree) {
    return pt.Tree.matcher({
        "stmts:stmts": ({ stmts }) => pt.matchers.zero_or_more_matcher(stmts).map(stmt_matcher),
    })(tree);
}
exports.stmts_matcher = stmts_matcher;
function stmt_matcher(tree) {
    return pt.Tree.matcher({
        "stmt:def": ({ name, exp }) => Stmt.def(pt.Tree.str(name), exp_matcher(exp)),
        "stmt:claim": ({ claim, t, define, exp }, { span }) => {
            if (pt.Tree.str(claim) !== pt.Tree.str(define)) {
                throw new pt.ParsingError("Name mismatch.\n" +
                    `- name to claim  : ${pt.Tree.str(claim)}\n` +
                    `- name to define : ${pt.Tree.str(define)}\n`, { span });
            }
            return Stmt.def(pt.Tree.str(claim), Exp.the(exp_matcher(t), exp_matcher(exp)));
        },
    })(tree);
}
exports.stmt_matcher = stmt_matcher;
function properties_matcher(tree) {
    return pt.Tree.matcher({
        "properties:properties": ({ properties }) => new Map(pt.matchers.zero_or_more_matcher(properties).map(property_matcher)),
    })(tree);
}
exports.properties_matcher = properties_matcher;
function property_matcher(tree) {
    return pt.Tree.matcher({
        "property:property": ({ name, exp }) => [
            pt.Tree.str(name),
            exp_matcher(exp),
        ],
    })(tree);
}
exports.property_matcher = property_matcher;
function sat_matcher(tree) {
    return pt.Tree.matcher({
        "sat:sat": ({ entries }) => pt.matchers.zero_or_more_matcher(entries).map(sat_entry_matcher),
    })(tree);
}
exports.sat_matcher = sat_matcher;
function sat_entry_matcher(tree) {
    return pt.Tree.matcher({
        "sat_entry:sat_entry": ({ name, t, exp }) => ({
            name: pt.Tree.str(name),
            t: exp_matcher(t),
            exp: exp_matcher(exp),
        }),
    })(tree);
}
exports.sat_entry_matcher = sat_entry_matcher;
function scope_matcher(tree) {
    return pt.Tree.matcher({
        "scope:scope": ({ entries }) => pt.matchers.one_or_more_matcher(entries).map(scope_entry_matcher),
    })(tree);
}
exports.scope_matcher = scope_matcher;
function scope_entry_matcher(tree) {
    return pt.Tree.matcher({
        "scope_entry:scope_entry": ({ name, t }) => ({
            name: pt.Tree.str(name),
            t: exp_matcher(t),
        }),
    })(tree);
}
exports.scope_entry_matcher = scope_entry_matcher;
