import * as Closure from "./closure";
import * as Telescope from "./telescope";
import * as DelayedSums from "./delayed-sums";
import * as Neutral from "../neutral";
export declare type Value = pi | fn | case_fn | cls | obj | equal | same | absurd | str | quote | union | type_constructor | datatype | data_constructor | data | type | not_yet;
export interface pi {
    kind: "Value.pi";
    arg_t: Value;
    ret_t_cl: Closure.Closure;
}
export declare const pi: (arg_t: Value, ret_t_cl: Closure.Closure) => pi;
export interface fn {
    kind: "Value.fn";
    ret_cl: Closure.Closure;
}
export declare const fn: (ret_cl: Closure.Closure) => fn;
export interface case_fn {
    kind: "Value.case_fn";
    ret_cls: Array<Closure.Closure>;
}
export declare const case_fn: (ret_cls: Array<Closure.Closure>) => case_fn;
export interface cls {
    kind: "Value.cls";
    sat: Array<{
        name: string;
        t: Value;
        value: Value;
    }>;
    tel: Telescope.Telescope;
}
export declare const cls: (sat: Array<{
    name: string;
    t: Value;
    value: Value;
}>, tel: Telescope.Telescope) => cls;
export interface obj {
    kind: "Value.obj";
    properties: Map<string, Value>;
}
export declare const obj: (properties: Map<string, Value>) => obj;
export interface equal {
    kind: "Value.equal";
    t: Value;
    from: Value;
    to: Value;
}
export declare const equal: (t: Value, from: Value, to: Value) => equal;
export interface same {
    kind: "Value.same";
}
export declare const same: same;
export interface absurd {
    kind: "Value.absurd";
}
export declare const absurd: absurd;
export interface str {
    kind: "Value.str";
}
export declare const str: str;
export interface quote {
    kind: "Value.quote";
    str: string;
}
export declare const quote: (str: string) => quote;
export interface union {
    kind: "Value.union";
    left: Value;
    right: Value;
}
export declare const union: (left: Value, right: Value) => union;
export interface type_constructor {
    kind: "Value.type_constructor";
    name: string;
    t: Value;
    delayed: DelayedSums.DelayedSums;
}
export declare const type_constructor: (name: string, t: Value, delayed: DelayedSums.DelayedSums) => type_constructor;
export interface datatype {
    kind: "Value.datatype";
    type_constructor: type_constructor;
    args: Array<Value>;
    t: Value;
}
export declare const datatype: (type_constructor: type_constructor, args: Array<Value>, t: Value) => datatype;
export interface data_constructor {
    kind: "Value.data_constructor";
    type_constructor: type_constructor;
    tag: string;
    t: Value;
}
export declare const data_constructor: (type_constructor: type_constructor, tag: string, t: Value) => data_constructor;
export interface data {
    kind: "Value.data";
    data_constructor: data_constructor;
    args: Array<Value>;
    t: Value;
}
export declare const data: (data_constructor: data_constructor, args: Array<Value>, t: Value) => data;
export interface type {
    kind: "Value.type";
}
export declare const type: type;
export interface not_yet {
    kind: "Value.not_yet";
    t: Value;
    neutral: Neutral.Neutral;
}
export declare const not_yet: (t: Value, neutral: Neutral.Neutral) => not_yet;
