"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.description = exports.aliases = exports.command = void 0;
const Mod = __importStar(require("../../mod"));
const EarleyParser = __importStar(require("../../earley-parser"));
const errors_1 = require("../../errors");
const TableLexer = __importStar(require("../../table-lexer"));
const Span = __importStar(require("../../span"));
const lexers = __importStar(require("../../lexers"));
const ut = __importStar(require("../../../ut"));
const fs_1 = __importDefault(require("fs"));
const strip_ansi_1 = __importDefault(require("strip-ansi"));
exports.command = "parse <input>";
exports.aliases = ["$0"];
exports.description = "Parse text to tree by grammar";
exports.builder = {
    output: { type: "string", alias: "o" },
    grammar: { type: "string", demandOption: true },
    table: { type: "string" },
    nocolor: { type: "boolean", default: false },
};
exports.handler = async (argv) => {
    const lexer = argv.table
        ? TableLexer.build(await ut.read_object(argv.table))
        : lexers.common;
    const text = await fs_1.default.promises.readFile(argv.input, "utf8");
    const tokens = lexer.lex(text);
    const mod = Mod.build(await ut.read_object(argv.grammar));
    if (mod.metadata.start) {
        const grammar = Mod.dot(mod, mod.metadata.start);
        const parser = EarleyParser.create(grammar);
        try {
            const tree = parser.parse(tokens);
            ut.write_object(tree, argv.output);
        }
        catch (error) {
            if (error instanceof errors_1.ParsingError) {
                let message = error.message;
                message += "\n";
                message += Span.report(error.span, text);
                console.error(argv.nocolor ? strip_ansi_1.default(message) : message);
                process.exit(1);
            }
            else {
                throw error;
            }
        }
    }
    else {
        throw new Error(`No start mod.metadata: ${ut.inspect(mod.metadata)}`);
    }
};
