"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.harvest = void 0;
const Schedule = __importStar(require("../schedule"));
const TaskChart = __importStar(require("../task-chart"));
const Task = __importStar(require("../task"));
const errors_1 = require("../../errors");
const Value = __importStar(require("../../value"));
const Tree = __importStar(require("../../tree"));
const ut = __importStar(require("../../../ut"));
function harvest(schedule) {
    const start = 0;
    const end = TaskChart.length(schedule.chart) - 1;
    return harvest_node(schedule, schedule.grammar, start, end);
}
exports.harvest = harvest;
function harvest_node(schedule, grammar, start, end) {
    for (const task of TaskChart.tasks_at(schedule.chart, end)) {
        if (task_ripe_p(task)) {
            return Tree.node({ name: grammar.name, kind: task.choice_name }, harvest_body(schedule, get_parts(grammar, task.choice_name), task.progress, start), span_from_tokens(schedule.tokens, start, end));
        }
    }
    throw Schedule.parsing_error(schedule, grammar, start, end);
    function task_ripe_p(task) {
        return (start === task.index &&
            end === Task.progress_index(task) &&
            task.progress.length === task.parts.length &&
            Task.match_grammar_p(task, grammar));
    }
    function get_parts(grammar, choice_name) {
        const choices = Value.DelayedChoices.force(grammar.delayed);
        const parts = choices.get(choice_name);
        if (parts === undefined) {
            const span = span_from_tokens(schedule.tokens, start, end);
            throw new errors_1.ParsingError(`Can not find choice: ${choice_name}\n` +
                `grammar: ${ut.inspect(Value.present(grammar))}\n`, { span });
        }
        return parts;
    }
}
function harvest_body(schedule, parts, progress, start) {
    if (parts.length !== progress.length) {
        const span = span_from_tokens(schedule.tokens, start, start);
        throw new errors_1.ParsingError("[Schedule.harvest_body] Internal error\n" +
            "parts.length !== progress.length\n" +
            `parts.length: ${parts.length}\n` +
            `progress.length: ${progress.length}\n`, { span });
    }
    const body = {};
    let index = start;
    for (let i = 0; i < parts.length; i++) {
        const entry = progress[i];
        const part = parts[i];
        if (part.name) {
            body[part.name] = harvest_value(schedule, part.value, entry, index);
        }
        index = entry.index;
    }
    return body;
}
function harvest_value(schedule, value, entry, index) {
    if (entry.choice_name) {
        if (value.kind === "Value.grammar") {
            return harvest_node(schedule, value, index, entry.index);
        }
        else {
            const span = span_from_tokens(schedule.tokens, index, entry.index);
            throw new errors_1.ParsingError("The value should be Value.grammar.\n" +
                `value: ${ut.inspect(Value.present(value))}\n`, { span });
        }
    }
    else {
        const token = schedule.tokens[entry.index - 1];
        return Tree.leaf(token);
    }
}
function span_from_tokens(tokens, start, end) {
    if (tokens.length === 0)
        return { lo: 0, hi: 0 };
    if (start === end) {
        const token = tokens[start] || tokens[tokens.length - 1];
        return token.span;
    }
    else {
        const lo = tokens[start].span.lo;
        const hi = tokens[end - 1].span.hi;
        return { lo, hi };
    }
}
