"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.choice_present = exports.present = void 0;
function present(exp) {
    switch (exp.kind) {
        case "Exp.v": {
            const { name } = exp;
            return name;
        }
        case "Exp.fn": {
            const { name, ret } = exp;
            return { $fn: [name, present(ret)] };
        }
        case "Exp.ap": {
            const { target, args } = exp;
            return { $ap: [present(target), ...args.map(present)] };
        }
        case "Exp.str": {
            const { value } = exp;
            return JSON.stringify(value);
        }
        case "Exp.pattern": {
            const { label, value } = exp;
            return value.flags
                ? { $pattern: [label, value.source, value.flags] }
                : { $pattern: [label, value.source] };
        }
        case "Exp.grammar":
            const { name, choices } = exp;
            const result = {};
            for (const [choice_name, parts] of choices) {
                Object.assign(result, choice_present(name, choice_name, parts));
            }
            return result;
    }
}
exports.present = present;
function choice_present(grammar_name, choice_name, parts) {
    return {
        [`${grammar_name}:${choice_name}`]: parts.map((part) => {
            const { name, value } = part;
            return name ? { [name]: present(value) } : present(value);
        }),
    };
}
exports.choice_present = choice_present;
