"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.non_greedy = exports.at_least_at_most = exports.at_least = exports.exactly = exports.optional = exports.one_or_more = exports.zero_or_more = void 0;
const create_regexp_1 = require("../create-regexp");
function zero_or_more(re) {
    re = create_regexp_1.create_regexp(re);
    return new RegExp(re.source + "*");
}
exports.zero_or_more = zero_or_more;
function one_or_more(re) {
    re = create_regexp_1.create_regexp(re);
    return new RegExp(re.source + "+");
}
exports.one_or_more = one_or_more;
function optional(re) {
    re = create_regexp_1.create_regexp(re);
    return new RegExp(re.source + "?");
}
exports.optional = optional;
function exactly(n, re) {
    re = create_regexp_1.create_regexp(re);
    if (Number.isInteger(n) && n > 0) {
        return new RegExp(re.source + `{${n}}`);
    }
    else {
        throw new Error("expecting positive integer");
    }
}
exports.exactly = exactly;
function at_least(n, re) {
    re = create_regexp_1.create_regexp(re);
    if (Number.isInteger(n) && n > 0) {
        return new RegExp(re.source + `{${n},}`);
    }
    else {
        throw new Error("expecting positive integer");
    }
}
exports.at_least = at_least;
function at_least_at_most(n, m, re) {
    re = create_regexp_1.create_regexp(re);
    if (Number.isInteger(n) && n >= 0 && Number.isInteger(m) && m > n) {
        return new RegExp(re.source + `{${n},${m}}`);
    }
    else {
        throw new Error("expecting n to be zero or positive integer and m > n\n" +
            `n: ${n}\n` +
            `m: ${m}\n`);
    }
}
exports.at_least_at_most = at_least_at_most;
function non_greedy(re) {
    re = create_regexp_1.create_regexp(re);
    if (re.source.length === 0) {
        throw new Error("Empty RegExp source");
    }
    else if (re.source[re.source.length - 1] === "*" ||
        re.source[re.source.length - 1] === "+" ||
        re.source[re.source.length - 1] === "?" ||
        re.source[re.source.length - 1] === "}") {
        return new RegExp(re.source + "?");
    }
    else {
        throw new Error("Expecting quantifier\n" + `source: ${re.source}\n`);
    }
}
exports.non_greedy = non_greedy;
