"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const rr = __importStar(require("../index"));
const ut = __importStar(require("../../ut"));
// NOTE
//   https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Regular_Expressions/Unicode_Property_Escapes
const sentence = "A ticket to 大阪 costs ¥2000 👌.";
{
    const emoji_presentation = /\p{Emoji_Presentation}/;
    const re = rr.add_flag(emoji_presentation, rr.flags.global + rr.flags.unicode);
    assert_1.default(ut.equal(sentence.match(re), ["👌"]));
}
{
    const non_latin = rr.one_or_more(/\P{Script_Extensions=Latin}/);
    const re = rr.add_flag(non_latin, rr.flags.global + rr.flags.unicode);
    assert_1.default(ut.equal(sentence.match(re), [" ", " ", " 大阪 ", " ¥2000 👌."]));
}
{
    const currency_or_punctuation = rr.or(/\p{Currency_Symbol}/, /\p{Punctuation}/);
    const re = rr.add_flag(currency_or_punctuation, rr.flags.global + rr.flags.unicode);
    assert_1.default(ut.equal(sentence.match(re), ["¥", "."]));
}
{
    // NOTE General_Category
    // finding all the letters of a text
    let story = "It’s the Cheshire Cat: now I shall have somebody to talk to.";
    // Most explicit form
    const letter = /\p{General_Category=Letter}/;
    const re = rr.add_flag(letter, rr.flags.global + rr.flags.unicode);
    assert_1.default(story.match(re));
    assert_1.default(re.exec(story));
}
{
    // NOTE Script
    const mixed = "aεЛ汉漢";
    assert_1.default(ut.equal(mixed.match(/\p{Script=Latin}/gu), ["a"]));
    assert_1.default(ut.equal(mixed.match(/\p{Script=Greek}/gu), ["ε"]));
    assert_1.default(ut.equal(mixed.match(/\p{Script=Cyrillic}/gu), ["Л"]));
    assert_1.default(ut.equal(mixed.match(/\p{Script=Han}/gu), ["汉", "漢"]));
}
{
    // NOTE Unicode property escapes vs. character classes
    const non_english_text = "Приключения Алисы в Стране чудес";
    // Trying to use ranges to avoid \w limitations:
    const re = /([\u0000-\u0019\u0021-\uFFFF])+/gu;
    // BMP goes through U+0000 to U+FFFF but space is U+0020
    assert_1.default(ut.equal(non_english_text.match(re), [
        "Приключения",
        "Алисы",
        "в",
        "Стране",
        "чудес",
    ]));
    // Using Unicode property escapes instead
    const upe = /\p{Letter}+/gu;
    assert_1.default(ut.equal(non_english_text.match(upe), [
        "Приключения",
        "Алисы",
        "в",
        "Стране",
        "чудес",
    ]));
}
{
    // NOTE about chinese character
    const chinese_text = "山川壯麗，物產豐隆，炎黃世胄，東亞稱雄。";
    assert_1.default(ut.equal(chinese_text.match(/\p{Letter}+/gu), [
        "山川壯麗",
        "物產豐隆",
        "炎黃世胄",
        "東亞稱雄",
    ]));
    assert_1.default(ut.equal(chinese_text.match(/\p{Punctuation}+/gu), ["，", "，", "，", "。"]));
}
